@@ -91,11 +91,13 @@ class _OSLCOperations_Mixin:
     def do_complex_query(self,queryresource, *, querystring='', searchterms=None, select='', orderby='', properties=None, isnulls=None
                         ,isnotnulls=None, enhanced=True, show_progress=True
                         ,show_info=False, verbose=False, maxresults=None, delaybetweenpages=0.0
-                        , pagesize=200
+                        ,pagesize=200
                         ,resolvenames=True
                         ,totalize=False
                         ,saverawresults=None
+                        ,addcolumns=None
                      ):
+        addcolumns = addcolumns or {}
         if searchterms and querystring:
             logger.info( f"{searchterms=}" )
             logger.info( f"{querystring=}" )
@@ -173,6 +175,15 @@ class _OSLCOperations_Mixin:
         # go through the results, mapping attribute uris back to names
         mappedresult = {}
         originalresults = resultstack[0]
+        
+        # add requested columns to each result
+        if len(addcolumns)>0:
+#            print( f"Adding columns {addcolumns}" )
+            for k in originalresults.keys():
+#                print( f"{k=}" )
+                for k1,v1 in addcolumns.items():
+#                    print( f"Adding {k1=} {v1=}" )
+                    originalresults[k][k1] = v1
         remappednames = {}
 
         if verbose:
@@ -562,7 +573,6 @@ class _OSLCOperations_Mixin:
         if verbose:
             print( f"Full query URL is {fullurl}" )
 #        print( f"Full query URL is {fullurl}" )
-#        burp
         # retrieve all pages of results - they will be processed later
         total = 1
         page = 0
